/*
 * \file handler.c
 *
 * \date Feb 15, 2017
 * \author Ting
 */

#include "handler.h"
#include "ledRGB.h"
#include "lcdled.h"
#include "estrutura.h"

/*! 
 * vari&aacute;vel de controle da contagem de tempo (m&uacute;ltiplo de 0.25s)
 */
extern uint32_t tempo;    

/*!
 * vari&aacute;vel de cor selecionada
 */         
extern uint8_t indice;            

/*!
 * vari&aacute;vel de estado da botoeira PTA12 (1: pressionado; 0 solto)
 */
extern uint8_t estado_pta12;     

/*!
 * varivel de estado do ledRGB
 */
extern uint8_t apagado;

/*!
 * vetor de cores do led
 */
extern cor cores_estaticas[];
/*!
 * buffer circular de recep&ccedil;&atilde;o
 */
extern buffer_circular buffer_receptor;

/*!
 * buffer circular de transmiss&atilde;o
 */
extern buffer_circular buffer_transmissor;

/*!
 * \fn PORTA_IRQHandler (void)
 * \brief Rotina de servio para GPIOA (Vetor 46, IRQ30, prioridade em NVICIPR7[31:30])
 */
void PORTA_IRQHandler(void) {
	static uint32_t tempo1=0, tempo2=0;
	float m;
	
	if (PORTA_PCR5 & PORT_PCR_ISF_MASK) {
		PORTA_PCR5 |= PORT_PCR_ISF_MASK;            ///< abaixa a bandeira de interrup&ccedil;&atilde;o
	} else if (PORTA_PCR12 & PORT_PCR_ISF_MASK) {   
		if (GPIOA_PDIR & GPIO_PIN(12)) {
			/*! Soltou */
			estado_pta12 = 0;
			tempo2 = SYST_CVR;
	        if (tempo2 > tempo1) m = (tempo2-tempo1)/((float)(SYST_RVR));
	        else m = (SYST_RVR-tempo1+tempo2)/((float)(SYST_RVR));
			indice = (tempo > 0)?(tempo+(SYST_RVR-tempo1+tempo2)/((float)SYST_RVR)):(int)m;   ///< computa tempo transcorrido em mltiplo de 0.25s
			// SYST_CSR &= ~SysTick_CSR_ENABLE_MASK;    ///< desabilita o clock do SysTick
            enviaLed(0x00);               ///< apaga leds vermelhos
		} else {
			/*! Apertou */
            enviaLed(0xFF);               ///< liga leds vermelhos 
			estado_pta12 = 1;
			tempo1 = SYST_CVR;
			tempo = 0;         ///< inicia a contagem de tempo
			// SYST_CSR |= SysTick_CSR_ENABLE_MASK;     ///< habilita o clock do SysTick       
		}
		PORTA_PCR12 |= PORT_PCR_ISF_MASK;            ///< abaixa a bandeira de interrup&ccedil;&atilde;o
	} 
}

/*!
 * \fn SysTick_Handler (void)
 * \brief Rotina de servio para exce&ccedil&atilde;o SysTick (Vetor 15)
 */
void SysTick_Handler(void) {
	tempo++;	
}

/**
 * \fn PIT_IRQHandler (void)
 * 
 * \brief Rotina de servio para PIT (Vetor 38, IRQ22, prioridade em NVICIPR5[23:22])
 * 
 */
void PIT_IRQHandler(void) {
	//GPIOD_PTOR |= GPIO_PTOR_PTTO(GPIO_PIN(1));  ///< GPIOD_PTOR[1]=1 (alterna o valor)
    /*!
     * Exibe a cor piscante conforme o intervalo de tempo estimado
     */
	if (apagado) {
		apagaLedRGB();
		apagado = 0;
	} else {
		led (VERMELHO, *((estado_led *)(cores_estaticas[indice].vermelho)));
		led (VERDE, *((estado_led *)(cores_estaticas[indice].verde)));
		led (AZUL, *((estado_led *)(cores_estaticas[indice].azul)));
		apagado = 1;
	}

	PIT_TFLG0 |= PIT_TFLG_TIF_MASK;     ///< TIF=PIT_TFLG0[31]: w1c (limpa pendncias)
}

/**
 * \brief non-maskable interrupt handler
 * \note A fun&ccedil;&atilde;o do PTA4 &eacute; NMI. Se habilitar no NVIC o processamento
 * da interrup&ccedil&atilde;o da porta A, pode-se gerar interrup&ccedil;&otilde;es inesperadas.
 * Uma solu&ccedil;&atilde;o &eacute; criar uma rotina de tratamente vazia 
 * (entra e sai sem fazer nada). 
 */
void NMI_Handler (void) {
}

/*!
 * \fn UART0_IRQHandler (void)
 * \brief Rotina de servio para UART0 (Vetor 28)
 */
void UART0_IRQHandler(void) {
	char d;

	/*!
	 * S&oacute; h&aacute; uma exce&ccedil;&atilde;o associada ao UARTx.
	 * Por&eacute;m cada m&oacute;dulo UARTx disp&oacute;e de registradores de estado 
	 * que indicam diferentes condi&ccedil;&otilde;es de exce&ccedil;&atilde;o.
	 */
	if (UART0_S1 & UART0_S1_RDRF_MASK)          
	{
		/*!
		 * Evento gerado pelo receptor
		 */
		d = UART0_D;                 
		/*!
		 * insere o item no buffer circular
		 */
		while (!bufCircular_insere (&buffer_receptor, d));
	} else if (UART0_S1 & UART0_S1_TDRE_MASK) {
		/*!
		 * Evento gerado pelo transmissor
		 */
		if (bufCircular_vazia(&buffer_transmissor))
			UART0_C2 &= ~UART_C2_TIE_MASK;
		else 
			UART0_D = bufCircular_remove(&buffer_transmissor);
	}	
}

/*!
 * \fn FTM0_IRQHandler (void)
 * \brief Rotina de servio para TPM0 (Vetor 33, IRQ17, prioridade em NVICIPR4[15:14])
 * Acender o segundo led vermelho do shield FEEC
 */
void FTM0_IRQHandler(void) {
	
  if (TPM0_STATUS & TPM_STATUS_CH0F_MASK) {
	TPM0_C0SC |= TPM_CnSC_CHF_MASK;       	///< CHF=TPM0_C0SC[7]: w1c (limpa a pend&ecirc;ncia do canal) 
	TPM0_C0SC &= ~TPM_CnSC_CHIE_MASK;     	///< desabilita a interrup&ccedil;&atilde;o
	/*!
	 * Libera o sinal do pino para o led vermelho com um pulso de enable
	 */
	GPIOC_PSOR = xxxx;     		        ///< Seta 1 no PORTC[10]
	GPIOC_PCOR = xxxx;       		///< Reseta o PORTC[10]
  }
}

/*!
 * \brief Rotina de servi&ccedil;o para TPM1 (Vetor 34, IRQ18, prioridade em NVICIPR4[23:22]).
 * Captar ambas bordas e registra os instantes (valor do contador) em que ocorreram. 
 */
void FTM1_IRQHandler(void) {
	static unsigned int inicio, fim;
	static uint32_t tof1_vezes;
	unsigned int multiplos;
	
	if ((TPM1_STATUS & TPM_STATUS_TOF_MASK) && (TPM1_SC & TPM_SC_TOIE_MASK)) {
		tof1_vezes++;
		TPM1_STATUS |= TPM_STATUS_TOF_MASK;     ///< TOF=TPM1_STATUS[8]: w1c (limpa a pend&ecirc;ncia do canal)
	} else if (TPM1_STATUS & TPM_STATUS_CH0F_MASK) {
		if (GPIOA_PDIR & GPIO_PIN(12)) { ///< soltou PTA12
			fim = TPM1_C0V;                         ///< captura o instante em que soltou
			multiplos = xxxx;      ///< m&uacute;ltiplos de 0.25ms
	    	if (multiplos > 12) indice = 12;
	    	else indice = multiplos;
	    	
			/*!
			 * Desliga led vermelho do shield FEEC
			 */
			reconfigModoTPM0Cn(0,xxxx);          ///< modo "clear output on match" 
	        NVIC_ICPR |= (1 << (33-16) );           ///< NVIC_ICPR[17]=1 (limpa as pend&ecirc;ncias do TPM0)
			enableTPM0CnInterrup(0);                ///< habilita interrup&ccedil;&atilde;o 
			reconfigValorTPM0Cn(0,TPM0_CNT+0x5);    ///< 5 unidades a mais em rela&ccedil;&atilde; ao valor corrente
	    	disableTPMInterrup(1);                   ///< desabilita timer overflow flag		
		} else {     ///< apertou PTA12
			inicio = xxxx;
			tof1_vezes=0; 
			enableTPMInterrup(1);  					///< habilita timer overflow flag
			/*!
			 * Liga led vermelho do shield FEEC
			 */
			reconfigModoTPM0Cn(0,xxxx);          ///< modo "set output on match" 
			reconfigValorTPM0Cn(0,TPM0_CNT+0x5);    ///< 5 unidades a mais em rela&ccedil;&atilde; ao valor corrente
			enableTPM0CnInterrup(0);                ///< habilita interrup&ccedil;&atilde;o 
		}
		TPM1_C0SC |= TPM_CnSC_CHF_MASK;     		///< CHF=TPM1_C1SC[7]: w1c (limpa a pend&ecirc;ncia do canal)
	}
}
